function hasOwnProperty(obj, prop) {
    var proto = obj.__proto__ || obj.constructor.prototype;
    return (prop in obj) &&
        (!(prop in proto) || proto[prop] !== obj[prop]);
}

var EntityManager = {
	entities: [],
	
	addEntity: function(entity) {
		this.entities.push(entity);
	},
	clearEntities: function() {
		this.entities = [];
	},
	
	update: function() {
		for (var i = 0; i < this.entities.length; i++) {
			if (typeof this.entities[i].update === 'function') {
				this.entities[i].update(this.entities);
			}
		}
	},
	render: function(ctx) {
		for (var i = 0; i < this.entities.length; i++) {
			if (typeof this.entities[i].render === 'function') {
				this.entities[i].render(ctx);
			}
		}
	},
	kill: function(entity) {
		var index = this.entities.indexOf(entity);
		if (index != -1) {
			for (var i = 0; i < entity.onKill.length; i++) {
				entity.onKill[i]();
			}
			this.entities.splice(index, 1);
		}
	},
	
	getEntitiesByName: function(name) {
		var result = [];
		for (var i = 0; i < this.entities.length; i++) {
			if (this.entities[i].name === name) {
				result.push(this.entities[i]);
			}
		}
		return result;
	},
	getClosestEntityByName: function(baseEntity, name, params = {}) {
		var closest = undefined;
		var closestDist = undefined;
		for (var i = 0; i < this.entities.length; i++) {
			var entity = this.entities[i];
			if (entity == this) continue;
			if (entity.name !== name) continue;
			
			var correctParams = true;
			for (var prop in params) {
				if (hasOwnProperty(entity, prop)) {
					if (entity[prop] != params[prop]) {
						correctParams = false;
					}
				}
			}
			if (!correctParams) continue;
			
			if (closest == undefined) {
				closest = entity;
				closestDist = baseEntity.distanceTo(closest);
			} else {
				var dist = baseEntity.distanceTo(closest);
				if (dist < closestDist) {
					closest = entity;
					closestDist = dist;
				}
			}
		}
		return closest;
	}
};

function Entity(name, img, x, y, w, h) {
	this.name = name;
	this.img = img;
	this.imgColor = undefined;
	this.x = x;
	this.y = y;
	this.w = w;
	this.h = h;
	this.rotation = 0;
	this.rotationOffset = 0;
	this.onKill = [];
	this.physical = true;
	this.lifeTime = -1;
}
Entity.prototype.addOnKill = function(callback) {
	this.onKill.push(callback);
};
Entity.prototype.distanceTo = function(other) {
	var dx = other.x - this.x;
	var dy = other.y - this.y;
	return (Math.sqrt(dx * dx) + (dy * dy));
};
Entity.prototype.angleTo = function(other) {
	var dx = other.x - this.x;
	var dy = other.y - this.y;
	var angle = (Math.atan2(dy, dx) * 180 / Math.PI);
	angle %= 360;
	if (angle < 0) angle += 360;
	return angle;
};
Entity.prototype.collideWith = function(other) {
	if (!this.physical || !other.physical) return false;
	var tl = this.x - this.w / 2;
	var tr = this.x + this.w / 2;
	var tt = this.y - this.h / 2;
	var tb = this.y + this.h / 2;
	
	var ol = other.x - other.w / 2;
	var or = other.x + other.w / 2;
	var ot = other.y - other.h / 2;
	var ob = other.y + other.h / 2;
	
	return !(tl > or || tr < ol || tt > ob || tb < ot);
};
Entity.prototype.update = function(entities) {
	if (this.lifeTime > 0) {
		this.lifeTime--;
		if (this.lifeTime == 0) {
			EntityManager.kill(this);
		}
	}
	
	if (this.img instanceof Animation) {
		this.img.update();
	}
};
Entity.prototype.render = function(ctx) {
	ctx.save();
	
	ctx.translate(this.x, this.y);
	ctx.rotate((this.rotation + this.rotationOffset) * Math.PI / 180);
	
	if (this.img instanceof Image) {
		ctx.drawImage(this.img, -this.w / 2, -this.h / 2, this.w, this.h);
	} else if (this.img instanceof Animation) {
		this.img.render(ctx, -this.w / 2, -this.h / 2, this.w, this.h);
	}
	
	if (this.imgColor != undefined) {
		ctx.globalCompositeOperation = "multiply";
		
		ctx.fillStyle = this.imgColor;
		ctx.fillRect(-this.w / 2, -this.h / 2, this.w, this.h);
		
		ctx.globalCompositeOperation = "source-over";
	}
	
	ctx.restore();
};

const EDGE_NONE = 0;
const EDGE_STOP = 1;
const EDGE_WRAP = 2;
const EDGE_KILL_DISCRETE = 3;
const EDGE_KILL_EXPLOSION = 4;

function MovableEntity(name, img, x, y, w, h) {
	Entity.call(this, name, img, x, y, w, h);
	this.xsp = 0;
	this.ysp = 0;
	this.horMode = EDGE_WRAP;
	this.topMode = EDGE_KILL_DISCRETE;
	this.bottomMode = EDGE_KILL_DISCRETE;
	this.gravity = true;
	this.gravityScale = 1;
};
MovableEntity.prototype = Object.create(Entity.prototype);
MovableEntity.prototype.setForwardSpeed = function(speed) {
	this.rotation %= 360;
	if (this.rotation < 0) this.rotation += 360;
	
	var rad = this.rotation * Math.PI / 180;
	this.xsp = Math.cos(rad) * speed;
	this.ysp = Math.sin(rad) * speed;
};
MovableEntity.prototype.rotateTowards = function(other, rotationSpeed) {
	var deltaAngle = this.angleTo(other) - this.rotation;
	var counterAngle;
	if (deltaAngle < 0) {
		counterAngle = 360 + deltaAngle;
	} else {
		counterAngle = 0 - (360 - deltaAngle);
	}
	if (Math.abs(deltaAngle) < Math.abs(counterAngle)) {
		if (deltaAngle > 0) {
			this.rotation += rotationSpeed;
		} else {
			this.rotation -= rotationSpeed;
		}
	} else {
		if (counterAngle > 0) {
			this.rotation += rotationSpeed;
		} else {
			this.rotation -= rotationSpeed;
		}
	}
};
MovableEntity.prototype.update = function(entities) {
	Entity.prototype.update.call(this, entities);
	
	this.x += this.xsp;
	this.y += this.ysp;
	
	if (this.gravity) {
		this.ysp += GRAVITY * this.gravityScale;
	}
	
	if (this.horMode == EDGE_STOP) {
		if (this.x < this.w / 2) {
			this.x = this.w / 2;
		} else if (this.x > (SCREEN_WIDTH - this.w / 2)) {
			this.x = SCREEN_WIDTH - this.w / 2;
		}
	} else if (this.horMode == EDGE_WRAP) {
		if (this.x < -(this.w / 2) - 1) {
			this.x = (SCREEN_WIDTH + this.w / 2) - 1;
		} else if (this.x > (SCREEN_WIDTH + this.w / 2 + 1)) {
			this.x = -(this.w / 2) + 1;
		}
	} else if (this.horMode == EDGE_KILL_DISCRETE || this.horMode == EDGE_KILL_EXPLOSION) {
		if (this.x < 0 || this.x > SCREEN_WIDTH) {
			if (this.horMode == EDGE_KILL_EXPLOSION) {
				EntityManager.addEntity(new Explosion(this.x, this.y));
			}
			EntityManager.kill(this);
		}
	}
	
	if (this.topMode == EDGE_STOP) {
		if (this.y < this.h / 2) {
			this.y = this.h / 2;
		}
	} else if (this.topMode == EDGE_KILL_DISCRETE || this.topMode == EDGE_KILL_EXPLOSION) {
		if (this.y < 0) {
			if (this.topMode == EDGE_KILL_EXPLOSION) {
				EntityManager.addEntity(new Explosion(this.x, this.y));
			}
			EntityManager.kill(this);
		}
	}
	
	if (this.bottomMode == EDGE_STOP) {
		if (this.y > SCREEN_HEIGHT - this.h / 2) {
			this.y = SCREEN_HEIGHT - this.h / 2;
		}
	} else if (this.bottomMode == EDGE_KILL_DISCRETE || this.bottomMode == EDGE_KILL_EXPLOSION) {
		if (this.y > SCREEN_HEIGHT) {
			if (this.bottomMode == EDGE_KILL_EXPLOSION) {
				EntityManager.addEntity(new Explosion(this.x, this.y));
			}
			EntityManager.kill(this);
		}	
	}
};
