function Plane(img, x, y) {
	MovableEntity.call(this, "Plane", img, x, y, 32, 32, 0);
	this.speed = 0;
	this.rotationSpeed = 2.75;
	this.control = 1;
	this.fullControlThreshold = 3;
	this.gain = 0.0425;
	this.started = false;
	this.stalled = false;
	this.friendly = true;
	
	this.horMode = EDGE_WRAP;
	this.topMode = EDGE_KILL_EXPLOSION;
	this.bottomMode = EDGE_KILL_EXPLOSION;
	
	this.maxRecentPoints = 60;
	this.recentPointFrequency = 2;
	this.recentPointTime = 0;
	this.recentPoints = [];
	
	this.gun = {
		projectile: {
			speed: 14
		},
		maxCooldown: 60,
		cooldown: 0,
		spread: 1,
		spreadAngle: 10,
		
		shoot: function() {
			if (this.canShoot()) {
				this.cooldown = this.maxCooldown;
			}
		},
		update: function() {
			this.cooldown--;
		},
		canShoot: function() {
			return (this.cooldown <= 0);
		}
	};
};
Plane.prototype = Object.create(MovableEntity.prototype);
Plane.prototype.liftOff = function(initialSpeed) {
	if (!this.started) {
		this.speed = initialSpeed;
		this.started = true;
	}
};
Plane.prototype.rotateLeft = function() {
	if (!this.started || this.stalled) return;
	this.rotation -= (this.rotationSpeed * this.control);
};
Plane.prototype.rotateRight = function() {
	if (!this.started || this.stalled) return;
	this.rotation += (this.rotationSpeed * this.control);
};
Plane.prototype.rotateTowards = function(other) {
	var deltaAngle = this.angleTo(other) - this.rotation;
	var counterAngle;
	if (deltaAngle < 0) {
		counterAngle = 360 + deltaAngle;
	} else {
		counterAngle = 0 - (360 - deltaAngle);
	}
	if (Math.abs(deltaAngle) < Math.abs(counterAngle)) {
		if (deltaAngle > 0) {
			this.rotateRight();
		} else {
			this.rotateLeft();
		}
	} else {
		if (counterAngle > 0) {
			this.rotateRight();
		} else {
			this.rotateLeft();
		}
	}
};
Plane.prototype.shoot = function() {
	if (this.gun.canShoot()) {
		this.gun.shoot();
		
		var currentAngle;
		if (this.gun.spread % 2 == 0) {
			currentAngle = -(this.gun.spreadAngle * this.gun.spread / 2) + this.gun.spreadAngle / 2;
		} else {
			currentAngle = -this.gun.spreadAngle * (Math.floor(this.gun.spread / 2));
		}
		for (var i = 0; i < this.gun.spread; i++) {
			var bulletRotation = this.rotation + currentAngle;
			var bullet = new Bullet(this.x, this.y, this.gun.projectile.speed, bulletRotation, this.friendly);
			EntityManager.addEntity(bullet);
			currentAngle += this.gun.spreadAngle;
		}
	}
};
Plane.prototype.update = function(entities) {
	if (!this.started) return;
	
	this.rotation %= 360;
	if (this.rotation < 0) this.rotation += 360;
	
	if (!this.stalled) {
		var rad = this.rotation * Math.PI / 180;
		var lift = Math.sin(rad);
		this.xsp = Math.cos(rad) * this.speed;
		this.ysp = lift * this.speed;
		
		this.speed += (lift * (this.gain * (lift < 0 ? 0.9475 : 1)));
		
		this.control = clamp(lerp(0, 1, this.speed / this.fullControlThreshold), 0, 1);
		if (this.control < 0.05) {
			this.stalled = true;
		}
	} else {
		if (this.rotation < 180) {
			this.rotation += this.rotationSpeed / 2;
		} else {
			this.rotation -= this.rotationSpeed / 2;
		}
		this.speed += GRAVITY;
		this.xsp = 0;
		this.ysp = this.speed;
	}
	
	MovableEntity.prototype.update.call(this, entities);
	
	this.gun.update();
	if (this.weapon != undefined) {
		var rad = this.rotation * Math.PI / 180;
		this.weapon.x = this.x + Math.cos(rad) * 20;
		this.weapon.y = this.y + Math.sin(rad) * 20;
	}
	
	if (this.recentPointTime > 0) {
		this.recentPointTime--;
	} else {
		this.recentPointTime = this.recentPointFrequency;
		if (this.recentPoints.length >= this.maxRecentPoints) {
			this.recentPoints.shift();
		}
		this.recentPoints.push({x: this.x, y: this.y});
	}
	
	for (var i = 0; i < entities.length; i++) {
		var other = entities[i];
		if (this == other) continue;
		if (this.collideWith(other)) {
			if (other.name === "Bullet" && this.friendly != other.friendly) {
				this.speed -= 0.5;
				EntityManager.kill(other);
			} else if (other.name === "Loot") {
				var flashString;
				if (other.power.speed != undefined) {
					this.speed += other.power.speed;
					flashString = "Speed+";
				}
				if (other.power.rotationSpeed != undefined) {
					this.rotationSpeed += other.power.rotationSpeed;
					flashString = "Control+";
				}
				if (other.power.gunSpread != undefined) {
					this.gun.spread += other.power.gunSpread;
					flashString = "Spread+";
				}
				if (other.power.gunMaxCooldown != undefined) {
					this.gun.maxCooldown += other.power.gunMaxCooldown;
					flashString = "Fire rate+";
				}
				if (other.power.gunProjectileSpeed != undefined) {
					this.gun.projectile.speed += other.power.gunProjectileSpeed;
					flashString = "Bullet speed+";
				}
				GUI.addFlash(flashString, this.x, this.y, (this.friendly ? FLASH_GOOD : FLASH_BAD));
				
				EntityManager.kill(other);
			}
		}
	}
};
Plane.prototype.render = function(ctx) {
	MovableEntity.prototype.render.call(this, ctx);
	
	ctx.lineWidth = 2;
	ctx.strokeStyle = (this.friendly ? "rgba(255,255,255,0.2)" : "rgba(255,0,0,0.2)");
	ctx.beginPath();
	for (var i = 0; i < this.recentPoints.length; i++) {
		var point = this.recentPoints[i];
		if (i > 0) {
			var prevPoint = this.recentPoints[i - 1];
			var dx = prevPoint.x - point.x;
			var dy = prevPoint.y - point.y;
			var dist = (Math.sqrt(dx * dx) + (dy * dy));
			if (dist > 600) {
				ctx.stroke();
				ctx.moveTo(point.x, point.y);
				ctx.beginPath();
			}
		}
		ctx.lineTo(point.x, point.y);
	}
	ctx.stroke();
};