const SCREEN_WIDTH = 960;
const SCREEN_HEIGHT = 640;

const FPS = 60;
const FRAME_WAIT = (1 / FPS) * 1000;

const GRAVITY = 0.065;

function lerp(v0, v1, t) {
	return (1-t)*v0 + t*v1;
}
function clamp(val, min, max) {
	return (val < min ? min : (val > max ? max : val));
}

const FLASH_GOOD = "#8F8";
const FLASH_BAD = "#E6E";

var GUI = {
	flashTime: 90,
	fadeTime: 10,
	flashes: []
};
GUI.update = function() {
	for (var i = 0; i < this.flashes.length; i++) {
		var flash = this.flashes[i];
		if (flash.life > 0) {
			flash.life--;
		} else {
			if (flash.fade > 0) {
				flash.fade--;
			} else {
				this.flashes.splice(i, 1);
			}
		}
	}
};
GUI.render = function(ctx) {
	ctx.textAlign = "center";
	ctx.font = "18px Arial";
	for (var i = 0; i < this.flashes.length; i++) {
		var flash = this.flashes[i];
		
		ctx.globalAlpha = (flash.fade / flash.maxFade);
		
		ctx.strokeStyle = "#000"
		ctx.lineWidth = 3;
		ctx.strokeText(flash.string, flash.x, flash.y);
		ctx.fillStyle = flash.color;
		ctx.fillText(flash.string, flash.x, flash.y);
		
		ctx.globalAlpha = 1.0;
	}
};
GUI.addFlash = function(string, x, y, color = "#000") {
	this.flashes.push({string: string, x: x, y: y, color: color, life: this.flashTime, maxFade: this.fadeTime, fade: this.fadeTime});
};


function Timer(maxTime, randomness = 0) {
	this.maxTime = maxTime;
	this.randomness = randomness;
	this.time = 0;
	this.started = false;
	this.onComplete = undefined;
};
Timer.prototype.update = function() {
	if (this.started) {
		if (this.time > 0) {
			this.time--;
		} else {
			if (typeof this.onComplete === 'function') {
				this.started = false;
				this.onComplete();
			}
		}
	}
};
Timer.prototype.start = function() {
	this.started = true;
	this.time = this.maxTime + Math.round(Math.random() * this.randomness);
};
Timer.prototype.stop = function() {
	this.started = false;
	this.time = 0;
};


function Animation(img, numFrames, xofs, yofs, frameWidth, frameHeight) {
	this.img = img;
	this.numFrames = numFrames;
	this.xofs = xofs;
	this.yofs = yofs;
	this.frameWidth = frameWidth;
	this.frameHeight = frameHeight;
	this.playing = false;
	this.frameDuration = 0;
	this.currentFrameIndex = 0;
	this.currentFrameTime = 0;
	this.onComplete = undefined;
};
Animation.prototype.play = function(frameDuration) {
	this.playing = true;
	this.frameDuration = frameDuration;
};
Animation.prototype.update = function() {
	if (this.currentFrameTime < this.frameDuration) {
		this.currentFrameTime++;
	} else {
		this.currentFrameTime = 0;
		this.currentFrameIndex++;
		if (this.currentFrameIndex >= this.numFrames) {
			this.currentFrameIndex = 0;
			if (typeof this.onComplete === 'function') {
				this.onComplete();
			}
		}
	}
};
Animation.prototype.render = function(ctx, x, y, w, h) {
	ctx.drawImage(this.img, this.xofs + this.frameWidth * this.currentFrameIndex, this.yofs, this.frameWidth, this.frameHeight, x, y, w, h);
};